@@ -72,8 +72,6 @@ class Parser extends EventEmitter {
     if (result) {
       this.packet.length = result.value
       this._list.consume(result.bytes)
-    } else {
-      this._emitError(new Error('Invalid length'))
     }
     debug('_parseLength %d', result.value)
     return !!result
@@ -160,6 +158,11 @@ class Parser extends EventEmitter {
 
     packet.protocolVersion = this._list.readUInt8(this._pos)
 
+    if (packet.protocolVersion >= 128) {
+      packet.bridgeMode = true
+      packet.protocolVersion = packet.protocolVersion - 128
+    }
+
     if (packet.protocolVersion !== 3 && packet.protocolVersion !== 4 && packet.protocolVersion !== 5) {
       return this._emitError(new Error('Invalid protocol version'))
     }
@@ -246,12 +249,17 @@ class Parser extends EventEmitter {
     debug('_parseConnack')
     const packet = this.packet
 
-    if (this._list.length < 2) return null
-
+    if (this._list.length < 1) return null
     packet.sessionPresent = !!(this._list.readUInt8(this._pos++) & constants.SESSIONPRESENT_MASK)
+
     if (this.settings.protocolVersion === 5) {
-      packet.reasonCode = this._list.readUInt8(this._pos++)
+      if (this._list.length >= 2) {
+        packet.reasonCode = this._list.readUInt8(this._pos++)
+      } else {
+        packet.reasonCode = 0
+      }
     } else {
+      if (this._list.length < 2) return null
       packet.returnCode = this._list.readUInt8(this._pos++)
     }
 
@@ -334,6 +342,10 @@ class Parser extends EventEmitter {
         subscription.nl = nl
         subscription.rap = rap
         subscription.rh = rh
+      } else if (this.settings.bridgeMode) {
+        subscription.rh = 0
+        subscription.rap = true
+        subscription.nl = true
       }
 
       // Push pair to subscriptions
@@ -421,6 +433,8 @@ class Parser extends EventEmitter {
         // response code
         packet.reasonCode = this._parseByte()
         debug('_parseConfirmation: packet.reasonCode `%d`', packet.reasonCode)
+      } else {
+        packet.reasonCode = 0
       }
 
       if (packet.length > 3) {
@@ -442,7 +456,11 @@ class Parser extends EventEmitter {
 
     if (this.settings.protocolVersion === 5) {
       // response code
-      packet.reasonCode = this._parseByte()
+      if (this._list.length > 0) {
+        packet.reasonCode = this._parseByte()
+      } else {
+        packet.reasonCode = 0
+      }
       // properies mqtt 5
       const properties = this._parseProperties()
       if (Object.getOwnPropertyNames(properties).length) {
@@ -542,6 +560,7 @@ class Parser extends EventEmitter {
 
   _parseVarByteNum (fullInfoFlag) {
     debug('_parseVarByteNum')
+    const maxBytes = 4
     let bytes = 0
     let mul = 1
     let value = 0
@@ -549,7 +568,7 @@ class Parser extends EventEmitter {
     let current
     const padding = this._pos ? this._pos : 0
 
-    while (bytes < 5) {
+    while (bytes < maxBytes && (padding + bytes) < this._list.length) {
       current = this._list.readUInt8(padding + bytes++)
       value += mul * (current & constants.VARBYTEINT_MASK)
       mul *= 0x80
@@ -563,6 +582,10 @@ class Parser extends EventEmitter {
       }
     }
 
+    if (!result && bytes === maxBytes && this._list.length >= bytes) {
+      this._emitError(new Error('Invalid variable byte integer'))
+    }
+
     if (padding) {
       this._pos += bytes
     }
