@@ -97,6 +97,26 @@ function testGenerateOnly (name, object, buffer, opts) {
   })
 }
 
+function testParseOnly (name, object, buffer, opts) {
+  test(name, t => {
+    const parser = mqtt.parser(opts)
+    // const expected = object
+    // const fixture = buffer
+
+    t.plan(2 + Object.keys(object).length)
+
+    parser.on('packet', packet => {
+      t.equal(Object.keys(object).length, Object.keys(packet).length, 'key count')
+      Object.keys(object).forEach(key => {
+        t.deepEqual(packet[key], object[key], `expected packet property ${key}`)
+      })
+    })
+
+    t.equal(parser.parse(buffer), 0, 'remaining bytes')
+    t.end()
+  })
+}
+
 function testParseError (expected, fixture, opts) {
   test(expected, t => {
     t.plan(1)
@@ -138,28 +158,14 @@ function testGenerateErrorMultipleCmds (cmds, expected, fixture, opts) {
   )
 }
 
-function testParseGenerateDefaults (name, object, buffer, opts) {
-  test(`${name} parse`, t => {
-    const parser = mqtt.parser(opts)
-    const expected = object
-    const fixture = buffer
+function testParseGenerateDefaults (name, object, buffer, generated, opts) {
+  testParseOnly(`${name} parse`, generated, buffer, opts)
+  testGenerateOnly(`${name} generate`, object, buffer, opts)
+}
 
-    t.plan(1 + Object.keys(expected).length)
-
-    parser.on('packet', packet => {
-      Object.keys(expected).forEach(key => {
-        t.deepEqual(packet[key], expected[key], `expected packet property ${key}`)
-      })
-    })
-
-    t.equal(parser.parse(fixture), 0, 'remaining bytes')
-    t.end()
-  })
-
-  test(`${name} generate`, t => {
-    t.equal(mqtt.generate(object).toString('hex'), buffer.toString('hex'))
-    t.end()
-  })
+function testParseAndGenerate (name, object, buffer, opts) {
+  testParseOnly(`${name} parse`, object, buffer, opts)
+  testGenerateOnly(`${name} generate`, object, buffer, opts)
 }
 
 function testWriteToStreamError (expected, fixture) {
@@ -289,6 +295,52 @@ testGenerateOnly('minimal connect with clientId as Buffer', {
   116, 101, 115, 116 // Client ID
 ]))
 
+testParseGenerate('connect MQTT bridge 131', {
+  cmd: 'connect',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 18,
+  protocolId: 'MQIsdp',
+  protocolVersion: 3,
+  bridgeMode: true,
+  clean: false,
+  keepalive: 30,
+  clientId: 'test'
+}, Buffer.from([
+  16, 18, // Header
+  0, 6, // Protocol ID length
+  77, 81, 73, 115, 100, 112, // Protocol ID
+  131, // Protocol version
+  0, // Connect flags
+  0, 30, // Keepalive
+  0, 4, // Client ID length
+  116, 101, 115, 116 // Client ID
+]))
+
+testParseGenerate('connect MQTT bridge 132', {
+  cmd: 'connect',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 18,
+  protocolId: 'MQIsdp',
+  protocolVersion: 4,
+  bridgeMode: true,
+  clean: false,
+  keepalive: 30,
+  clientId: 'test'
+}, Buffer.from([
+  16, 18, // Header
+  0, 6, // Protocol ID length
+  77, 81, 73, 115, 100, 112, // Protocol ID
+  132, // Protocol version
+  0, // Connect flags
+  0, 30, // Keepalive
+  0, 4, // Client ID length
+  116, 101, 115, 116 // Client ID
+]))
+
 testParseGenerate('connect MQTT 5', {
   cmd: 'connect',
   retain: false,
@@ -513,6 +565,76 @@ testParseGenerate('no clientId with 3.1.1', {
   0, 0 // Client ID length
 ]))
 
+testParseGenerateDefaults('no clientId with 5.0', {
+  cmd: 'connect',
+  protocolId: 'MQTT',
+  protocolVersion: 5,
+  clean: true,
+  keepalive: 60,
+  properties:
+    {
+      receiveMaximum: 20
+    },
+  clientId: ''
+}, Buffer.from(
+  [16, 16, 0, 4, 77, 81, 84, 84, 5, 2, 0, 60, 3, 33, 0, 20, 0, 0]
+), {
+  cmd: 'connect',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 16,
+  topic: null,
+  payload: null,
+  protocolId: 'MQTT',
+  protocolVersion: 5,
+  clean: true,
+  keepalive: 60,
+  properties: {
+    receiveMaximum: 20
+  },
+  clientId: ''
+}, { protocolVersion: 5 })
+
+testParseGenerateDefaults('utf-8 clientId with 5.0', {
+  cmd: 'connect',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 23,
+  protocolId: 'MQTT',
+  protocolVersion: 4,
+  clean: true,
+  keepalive: 30,
+  clientId: 'Ŧėśt🜄'
+}, Buffer.from([
+  16, 23, // Header
+  0, 4, // Protocol ID length
+  77, 81, 84, 84, // Protocol ID
+  4, // Protocol version
+  2, // Connect flags
+  0, 30, // Keepalive
+  0, 11, // Client ID length
+  197, 166, // Ŧ (UTF-8: 0xc5a6)
+  196, 151, // ė (UTF-8: 0xc497)
+  197, 155, // ś (utf-8: 0xc59b)
+  116, // t (utf-8: 0x74)
+  240, 159, 156, 132 // 🜄 (utf-8: 0xf09f9c84)
+]), {
+  cmd: 'connect',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 23,
+  topic: null,
+  payload: null,
+  protocolId: 'MQTT',
+  protocolVersion: 4,
+  clean: true,
+  keepalive: 30,
+  clientId: 'Ŧėśt🜄'
+}, { protocol: 5 })
+
 testParseGenerateDefaults('default connect', {
   cmd: 'connect',
   clientId: 'test'
@@ -520,7 +642,209 @@ testParseGenerateDefaults('default connect', {
   16, 16, 0, 4, 77, 81, 84,
   84, 4, 2, 0, 0,
   0, 4, 116, 101, 115, 116
-]))
+]), {
+  cmd: 'connect',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 16,
+  topic: null,
+  payload: null,
+  protocolId: 'MQTT',
+  protocolVersion: 4,
+  clean: true,
+  keepalive: 0,
+  clientId: 'test'
+})
+
+testParseAndGenerate('Version 4 CONACK', {
+  cmd: 'connack',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 2,
+  topic: null,
+  payload: null,
+  sessionPresent: false,
+  returnCode: 1
+}, Buffer.from([
+  32, 2, // Fixed Header (CONNACK, Remaining Length)
+  0, 1 // Variable Header (Session not present, Connection Refused - unacceptable protocol version)
+]), {}) // Default protocolVersion (4)
+
+testParseAndGenerate('Version 5 CONACK', {
+  cmd: 'connack',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 3,
+  topic: null,
+  payload: null,
+  sessionPresent: false,
+  reasonCode: 140
+}, Buffer.from([
+  32, 3, // Fixed Header (CONNACK, Remaining Length)
+  0, 140, // Variable Header (Session not present, Bad authentication method)
+  0 // Property Length Zero
+]), { protocolVersion: 5 })
+
+testParseOnly('Version 4 CONACK in Version 5 mode', {
+  cmd: 'connack',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 2,
+  topic: null,
+  payload: null,
+  sessionPresent: false,
+  reasonCode: 1 // a version 4 return code stored in the version 5 reasonCode because this client is in version 5
+}, Buffer.from([
+  32, 2, // Fixed Header (CONNACK, Remaining Length)
+  0, 1 // Variable Header (Session not present, Connection Refused - unacceptable protocol version)
+]), { protocolVersion: 5 }) // message is in version 4 format, but this client is in version 5 mode
+
+testParseOnly('Version 5 PUBACK test 1', {
+  cmd: 'puback',
+  messageId: 42,
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 2,
+  topic: null,
+  payload: null,
+  reasonCode: 0
+}, Buffer.from([
+  64, 2, // Fixed Header (PUBACK, Remaining Length)
+  0, 42 // Variable Header (2 Bytes: Packet Identifier 42, Implied Reason code: Success, Implied no properties)
+]), { protocolVersion: 5 }
+)
+
+testParseAndGenerate('Version 5 PUBACK test 2', {
+  cmd: 'puback',
+  messageId: 42,
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 3,
+  topic: null,
+  payload: null,
+  reasonCode: 0
+}, Buffer.from([
+  64, 3, // Fixed Header (PUBACK, Remaining Length)
+  0, 42, 0 // Variable Header (2 Bytes: Packet Identifier 42, Reason code: 0 Success, Implied no properties)
+]), { protocolVersion: 5 }
+)
+
+testParseOnly('Version 5 PUBACK test 3', {
+  cmd: 'puback',
+  messageId: 42,
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 4,
+  topic: null,
+  payload: null,
+  reasonCode: 0
+}, Buffer.from([
+  64, 4, // Fixed Header (PUBACK, Remaining Length)
+  0, 42, 0, // Variable Header (2 Bytes: Packet Identifier 42, Reason code: 0 Success)
+  0 // no properties
+]), { protocolVersion: 5 }
+)
+
+testParseOnly('Version 5 CONNACK test 1', {
+  cmd: 'connack',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 1,
+  topic: null,
+  payload: null,
+  sessionPresent: true,
+  reasonCode: 0
+}, Buffer.from([
+  32, 1, // Fixed Header (CONNACK, Remaining Length)
+  1 // Variable Header (Session Present: 1 => true, Implied Reason code: Success, Implied no properties)
+]), { protocolVersion: 5 }
+)
+
+testParseOnly('Version 5 CONNACK test 2', {
+  cmd: 'connack',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 2,
+  topic: null,
+  payload: null,
+  sessionPresent: true,
+  reasonCode: 0
+}, Buffer.from([
+  32, 2, // Fixed Header (CONNACK, Remaining Length)
+  1, 0 // Variable Header (Session Present: 1 => true, Connect Reason code: Success, Implied no properties)
+]), { protocolVersion: 5 }
+)
+
+testParseAndGenerate('Version 5 CONNACK test 3', {
+  cmd: 'connack',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 3,
+  topic: null,
+  payload: null,
+  sessionPresent: true,
+  reasonCode: 0
+}, Buffer.from([
+  32, 3, // Fixed Header (CONNACK, Remaining Length)
+  1, 0, // Variable Header (Session Present: 1 => true, Connect Reason code: Success)
+  0 // no properties
+]), { protocolVersion: 5 }
+)
+
+testParseOnly('Version 5 DISCONNECT test 1', {
+  cmd: 'disconnect',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 0,
+  topic: null,
+  payload: null,
+  reasonCode: 0
+}, Buffer.from([
+  224, 0 // Fixed Header (DISCONNECT, Remaining Length), Implied Reason code: Normal Disconnection
+]), { protocolVersion: 5 }
+)
+
+testParseOnly('Version 5 DISCONNECT test 2', {
+  cmd: 'disconnect',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 1,
+  topic: null,
+  payload: null,
+  reasonCode: 0
+}, Buffer.from([
+  224, 1, // Fixed Header (DISCONNECT, Remaining Length)
+  0 // reason Code (Normal disconnection)
+]), { protocolVersion: 5 }
+)
+
+testParseAndGenerate('Version 5 DISCONNECT test 3', {
+  cmd: 'disconnect',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 2,
+  topic: null,
+  payload: null,
+  reasonCode: 0
+}, Buffer.from([
+  224, 2, // Fixed Header (DISCONNECT, Remaining Length)
+  0, // reason Code (Normal disconnection)
+  0 // no properties
+]), { protocolVersion: 5 }
+)
 
 testParseGenerate('empty will payload', {
   cmd: 'connect',
@@ -1903,6 +2227,19 @@ testParseGenerate('disconnect MQTT 5', {
   28, 0, 4, 116, 101, 115, 116// serverReference
 ]), { protocolVersion: 5 })
 
+testParseGenerate('disconnect MQTT 5 with no properties', {
+  cmd: 'disconnect',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 2,
+  reasonCode: 0
+}, Buffer.from([
+  224, 2, // Fixed Header (DISCONNECT, Remaining Length)
+  0, // Reason Code (Normal Disconnection)
+  0 // Property Length (0 => No Properties)
+]), { protocolVersion: 5 })
+
 testParseGenerate('auth MQTT 5', {
   cmd: 'auth',
   retain: false,
